# The LoRa API

The LoRa API take place between a single _server_, which drive all LoRa related
devices, and multiple _clients_ wanting to interact with these devices. Its main
purpose is to provide abstraction of the LoRa devices to other process.

The LoRa API allows three ways of communication:
 - Receive data from the _server_ (**uplink**).
 - Send data to the _server_ (**downlink**).
 - Control the _server_ configuration (**control**).

The LoRa API defines a transport layer and a data exchange format.

## Transport layer

The transport layer is using [ZeroMQ][zeromq].

The transport layer provides three channels:
 - An **uplink** channel using a [publish-subscribe][zmq-pub-sub] pattern.
 - A **downlink** channel using a [push-pull][zmq-push-pull] pattern.
 - A **control** channel using a [request-reply][zmq-req-rep] pattern.

There is no restriction about channel's endpoint configuration. It is up to the
user to configure the _server_ and the _clients_ the same way.

## Data exchange format

The data exchange format is using [Google Protocol Buffers][protobuf].

Each channels can carry multiple message types. To be able to handle all this
message types, a unique container message (featuring the `oneof` keyword) is
used by channel.

### Uplink channel

The **uplink** channel can carry single and aggregated data messages. This way
the full channel can be configured for either speed or low traffic depending on
where the aggregation is made.

The **uplink** channel can carry:
[`proto::uplink::Container`](uplink-container.proto)
 - LoRa data messages (single, aggregated or splitted, with or without
   localization timing).
   [`proto::lora::Uplink`](uplink-lora.proto)
 - GFSK data messages (single, aggregated or splitted).
   [`proto::gfsk::Uplink`](uplink-gfsk.proto)
 - Event messages (system events, transmission status, ...).
   [`proto::event::Uplink`](uplink-event.proto)
 - Frequency scan data messages.
   [`proto::scan::Uplink`](uplink-scan.proto)

### Downlink channel

The **downlink** channel is only intended for data message transmission. The
data messages can be aggregated to provide multiple or alternative transmission
times and configurations. For each data message sent to the **downlink** channel
an event message must be received from the **uplink** channel to acknowledge the
transmission status.

The **downlink** channel can carry:
[`proto::downlink::Container`](downlink-container.proto)
 - LoRa data messages (single or aggregated).
   [`proto::lora::Downlink`](downlink-lora.proto)
 - GFSK data messages (single or aggregated).
   [`proto::gfsk::Downlink`](downlink-gfsk.proto)

### Control channel

The **control** channel is used with a [request-reply][zmq-req-rep] pattern.
So each request will lead to a reply. An _RPC_ protocol is implemented on the
channel using [`proto::control::Request`](control-rpc.proto) and
[`proto::control::Reply`](control-rpc.proto). They can be used to change the
_server_ configuration or to get information about it.

The **control** channel support these services:
 - Verbosity management:
   - The service `"set_verbosity"`
     takes [`proto::verbosity::Request`](control-verbosity.proto)
     and returns _nothing_.
 - Request for version:
   - The service `"get_version"`
     takes _nothing_
     and returns [`proto::version::Reply`](control-version.proto).
 - Frequency scan management (start and stop):
   - The service `"scan_status"`
     takes _nothing_
     and returns [`proto::scan::StatusReply`](control-scan.proto).
   - The service `"scan_start"`
     takes [`proto::scan::StartRequest`](control-scan.proto)
     and returns _nothing_.
   - The service `"scan_stop"`
     takes [`proto::scan::StopRequest`](control-scan.proto)
     and returns _nothing_.
 - Request for statistics (received and transmitted messages, ...):
   - The service `"get_statistics"`
     takes _nothing_
     and returns [`proto::statistic::Reply`](control-statistic.proto).

## Disclaimer

 1. All side-band related values (bandwidth ...) are expressed in double
    side-band (DSB).

## Todo list

 - Control channel:
   - Transmission RUD interface (read, update and delete).

## License

This work is released under the [3-Clause BSD License](LICENSE.md).

[zeromq]: http://zeromq.org
[zmq-pub-sub]: http://api.zeromq.org/4-2:zmq-socket#toc9
[zmq-push-pull]: http://api.zeromq.org/4-2:zmq-socket#toc14
[zmq-req-rep]: http://api.zeromq.org/4-2:zmq-socket#toc21
[protobuf]: https://developers.google.com/protocol-buffers
